@@ -1,7 +1,14 @@
-/* Hiawatha | cgi.c
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
  *
- * All the routines for handling CGI requests.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  */
+
 #include "config.h"
 #include <sys/types.h>
 #include <stdlib.h>
@@ -11,6 +18,7 @@
 #include <errno.h>
 #include <pthread.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -499,6 +507,7 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
  */
 int connect_to_fcgi_server(t_connect_to *connect_to) {
 	int sock = -1;
+	struct sockaddr_un sunix;
 	struct sockaddr_in saddr4;
 #ifdef HAVE_IPV6
 	struct sockaddr_in6 saddr6;
@@ -508,7 +517,18 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 		return -1;
 	}
 
-	if (connect_to->host.family == AF_INET) {
+	if (connect_to->unix_socket != NULL) {
+		/* UNIX socket
+		 */
+		if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) > 0) {
+			sunix.sun_family = AF_UNIX;
+			strncpy(sunix.sun_path, connect_to->unix_socket, (sizeof(struct sockaddr_un) - sizeof(short)));
+			if (connect(sock, (struct sockaddr*)&sunix, sizeof(struct sockaddr_un)) != 0) {
+				close(sock);
+				sock = -1;
+			}
+		}
+	} else if (connect_to->host.family == AF_INET) {
 		/* IPv4
 		 */
 		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) > 0) {
