@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #include "config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -119,6 +130,9 @@ void touch_logfiles(t_config *config, char *dir) {
 	if (config->garbage_logfile != NULL) {
 		touch_logfile(dir, config->garbage_logfile, 0640, config->server_uid, config->server_gid);
 	}
+	if (config->exploit_logfile != NULL) {
+		touch_logfile(dir, config->exploit_logfile, 0640, config->server_uid, config->server_gid);
+	}
 #ifdef DEBUG
 	touch_logfile(dir, LOG_DIR"/debug.log", 0640, config->server_uid, config->server_gid);
 #endif
@@ -805,7 +819,7 @@ int get_path_info(t_session *session) {
  */
 int serve_client(t_session *session) {
 	int result, length;
-	char *search, *conffile, *qmark, chr;
+	char *search, *conffile, *qmark, chr, *client_ip;
 	t_host *host_record;
 	t_access access;
 	t_deny_body *deny_body;
@@ -813,6 +827,7 @@ int serve_client(t_session *session) {
 	int i;
 	t_toolkit_options options;
 #endif
+	t_ip_addr ip;
 
 	if ((result = fetch_request(session)) != 200) {
 		session->request_method = GET;
@@ -822,6 +837,26 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
+	/* Hide reverse proxies
+	 */
+	if (in_iplist(session->config->hide_proxy, &(session->ip_address))) {
+		if ((client_ip = get_headerfield("X-Forwarded-For:", session->headerfields)) != NULL) {
+			if ((search = strrchr(client_ip, ',')) != NULL) {
+				client_ip = search + 1;
+			}
+
+			while ((*client_ip == ' ') && (*client_ip != '\0')) {
+				client_ip++;
+			}
+
+			if (*client_ip != '\0') {
+				if (parse_ip(client_ip, &ip) != -1) {
+					copy_ip(&(session->ip_address), &ip);
+				}
+			}
+		}
+	}
+
 	if (session->request_method == TRACE) {
 		if (session->binding->enable_trace == false) {
 			return 501;
@@ -863,10 +898,12 @@ int serve_client(t_session *session) {
 					ban_ip(&(session->ip_address), session->config->ban_on_denied_body, session->config->kick_on_ban);
 					log_system(session, "Client banned because of denied body");
 				}
+
 				log_string(session->host->error_logfile, "Request denied because of unallowed body content");
-				log_garbage(session);
+				log_exploit_attempt(session, "denied body", session->body);
 
 				*(session->body + session->content_length) = chr;
+
 				return 403;
 			}
 			deny_body = deny_body->next;
@@ -878,11 +915,7 @@ int serve_client(t_session *session) {
 #ifdef HAVE_TOOLKIT
 	/* URL toolkit
 	 */
-	options.new_url = NULL;
-	options.website_root = session->host->website_root;
-	options.fastcgi_server = NULL;
-	options.ban = 0;
-	options.url_toolkit = session->config->url_toolkit;
+	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit);
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
 		for (i = 0; i < session->host->toolkit_rules.size; i++) {
@@ -910,10 +943,15 @@ int serve_client(t_session *session) {
 				session->cause_of_301 = location;
 				return 301;
 			}
+
 			if (result == UT_DENY_ACCESS) {
 				log_error(session, "access denied via URL toolkit rule");
 				return 403;
 			}
+
+			if (options.expire > -1) {
+				session->expires = options.expire;
+			}
 		}
 	}
 #endif
