@@ -1,7 +1,14 @@
-/* Hiawatha | log.c
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
  *
- * Logging functions
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  */
+
 #include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -312,6 +319,10 @@ void log_garbage(t_session *session) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
+	if ((session->config->garbage_logfile == NULL) || (session->request == NULL)) {
+		return;
+	}
+
 	for (i = 0; i < session->bytes_in_buffer; i++) {
 		if (session->request[i] == '\0') {
 			if (spaces > 0) {
@@ -323,14 +334,35 @@ void log_garbage(t_session *session) {
 		}
 	}
 
-	if ((session->config->garbage_logfile != NULL) && (session->request != NULL)) {
-		if ((fp = fopen(session->config->garbage_logfile, "a")) != NULL) {
-			print_ip_address(str, &(session->ip_address));
-			print_timestamp(str + strlen(str));
-			fprintf(fp, "%s%s"EOL, str, session->request);
-			fclose(fp);
-		}
+	if ((fp = fopen(session->config->garbage_logfile, "a")) == NULL) {
+		return;
 	}
+
+	print_ip_address(str, &(session->ip_address));
+	print_timestamp(str + strlen(str));
+	fprintf(fp, "%s%s"EOL, str, session->request);
+	fclose(fp);
+}
+
+/* Log exploit attempt
+ */
+void log_exploit_attempt(t_session *session, char *type, char *data) {
+	FILE *fp;
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE], *host, *uri, *unknown = "unknown";
+
+	if ((session->config->exploit_logfile == NULL) || (type == NULL) || (data == NULL)) {
+		return;
+	} else if ((fp = fopen(session->config->exploit_logfile, "a")) == NULL) {
+		return;
+	}
+
+	host = (session->host->hostname.size > 0) ? session->host->hostname.item[0] : unknown;
+	uri = (session->uri != NULL) ? session->uri : unknown;
+
+	print_ip_address(str, &(session->ip_address));
+	print_timestamp(str + strlen(str));
+	fprintf(fp, "%s%s%s|%s|%s"EOL, str, host, uri, type, data);
+	fclose(fp);
 }
 
 /* Log an unbanning.
