@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #include "config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -36,6 +47,7 @@ typedef struct {
 	char       *pidfile;
 	char       *children;
 	char       *max_requests;
+	int        server_nr;
 	t_php_cgi  *php_cgi;
 	t_keyvalue *envir;
 } t_config;
@@ -273,6 +285,7 @@ t_config *read_config(char *config_file) {
 	config->pidfile = strdup(PIDFILE_DIR"/php-fcgi.pid");
 	config->children = strdup("3");
 	config->max_requests = strdup("100");
+	config->server_nr = -1;
 	config->php_cgi = NULL;
 	config->envir = NULL;
 
@@ -328,18 +341,143 @@ void show_help(char *php_cgi) {
 	printf("         -v: show version and exit.\n");
 }
 
+/* Start FastCGI servers
+ */
+int start_fastcgi_servers(t_config *config) {
+	FILE *fp;
+	t_keyvalue *env;
+	t_php_cgi *php_cgi;
+	int started = 0, status;
+
+	/* Start PHP FastCGI servers
+	 */
+	if (config->php_cgi == NULL) {
+		fprintf(stderr, "No servers have been defined.\n");
+		return -1;
+	}
+
+	clearenv();
+	setenv("PHP_FCGI_CHILDREN", config->children, 1);
+	setenv("PHP_FCGI_MAX_REQUESTS", config->max_requests, 1);
+	env = config->envir;
+	while (env != NULL) {
+		setenv(env->key, env->value, 1);
+		env = env->next;
+	}
+
+	if ((fp = fopen(config->pidfile, "r")) != NULL) {
+		fclose(fp);
+		fprintf(stderr, "A PID file exists. Are FastCGI daemons already running?\n");
+		return -1;
+	}
+
+	log_string("starting PHP FastCGI daemons");
+
+	/* Start daemons
+	 */
+	php_cgi = config->php_cgi;
+	while (php_cgi != NULL) {
+		if ((php_cgi->pid = run_php_cgi(php_cgi)) != -1) {
+			started++;
+		}
+		php_cgi = php_cgi->next;
+	}
+	if (started == 0) {
+		return -1;
+	}
+
+	/* Check return codes
+	 */
+	sleep(1);
+	php_cgi = config->php_cgi;
+	while (php_cgi != NULL) {
+		if (php_cgi->pid != -1) {
+			if (waitpid(php_cgi->pid, &status, WNOHANG) > 0) {
+				log_string("PHP FastCGI daemon exited with code %d", status);
+				php_cgi->pid = -1;
+			}
+		}
+		php_cgi = php_cgi->next;
+	}
+
+	/* Log PIDs
+	 */
+	if ((fp = fopen(config->pidfile, "w")) != NULL) {
+		php_cgi = config->php_cgi;
+		while (php_cgi != NULL) {
+			if (php_cgi->pid != -1) {
+				fprintf(fp, "%d\n", php_cgi->pid);
+			}
+			php_cgi = php_cgi->next;
+		}
+		fclose(fp);
+	} else {
+		fprintf(stderr, "write ");
+		perror(config->pidfile);
+
+		return -1;
+	}
+
+	return 0;
+}
+
+/* Stop running FastCGI servers
+ */
+int stop_fastcgi_servers(t_config *config) {
+	FILE *fp;
+	char line[11];
+	size_t len;
+	pid_t pid;
+
+	if ((fp = fopen(config->pidfile, "r")) != NULL) {
+		log_string("stopping PHP FastCGI daemons");
+
+		line[10] = '\0';
+		while (fgets(line, 10, fp) != NULL) {
+			if ((len = strlen(line)) > 0) {
+				if (line[len - 1] == '\n') {
+					line[len - 1] = '\0';
+				}
+				if ((pid = (pid_t)str2int(line)) > 1) {
+					if (kill(pid, SIGTERM) == -1) {
+						fprintf(stderr, "kill -15 ");
+						perror(line);
+					}
+				}
+			}
+		}
+
+		fclose(fp);
+		if (unlink(config->pidfile) == -1) {
+			fprintf(stderr, "unlink ");
+			perror(config->pidfile);
+
+			return -1;
+		}
+	} else {
+		fprintf(stderr, "read ");
+		perror(config->pidfile);
+
+		return -1;
+	}
+
+	return 0;
+}
+
 /* Main routine
  */
 int main(int argc, char *argv[]) {
 	t_config *config;
-	t_keyvalue *env;
-	t_php_cgi *php_cgi;
-	char *config_file = CONFIG_DIR"/php-fcgi.conf", line[11];
-	FILE *fp;
+	char *config_file = CONFIG_DIR"/php-fcgi.conf";
 	bool kill_servers = false;
-	pid_t pid;
-	size_t len;
-	int i = 0, started = 0, status;
+	int i = 0;
+
+	if ((config = read_config(config_file)) == NULL) {
+		return EXIT_FAILURE;
+	}
+	if (valid_config(config) == false) {
+		return EXIT_FAILURE;
+	}
 
 	while (++i < argc) {
 		if (strcmp(argv[i], "-c") == 0) {
@@ -356,6 +494,13 @@ int main(int argc, char *argv[]) {
 			kill_servers = true;
 		} else if (strcmp(argv[i], "-q") == 0) {
 			quiet = true;
+		} else if (strcmp(argv[i], "-s") == 0) {
+			if (++i < argc) {
+				config->server_nr = str2int(argv[i]);
+			} else {
+				fprintf(stderr, "Specify a server number.\n");
+				return EXIT_FAILURE;
+			}
 		} else if (strcmp(argv[i], "-v") == 0) {
 			printf("PHP-FastCGI v"VERSION"\n");
 			return EXIT_SUCCESS;
@@ -365,109 +510,13 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if ((config = read_config(config_file)) == NULL) {
-		return EXIT_FAILURE;
-	}
-
-	if (valid_config(config) == false) {
-		return EXIT_FAILURE;
-	}
-
 	if (kill_servers == false) {
-		/* Start PHP FastCGI servers
-		 */
-		if (config->php_cgi == NULL) {
-			fprintf(stderr, "No servers have been defined.\n");
+		if (start_fastcgi_servers(config) == -1) {
 			return EXIT_FAILURE;
 		}
-
-		clearenv();
-		setenv("PHP_FCGI_CHILDREN", config->children, 1);
-		setenv("PHP_FCGI_MAX_REQUESTS", config->max_requests, 1);
-		env = config->envir;
-		while (env != NULL) {
-			setenv(env->key, env->value, 1);
-			env = env->next;
-		}
-
-		if ((fp = fopen(config->pidfile, "r")) != NULL) {
-			fclose(fp);
-			fprintf(stderr, "A PID file exists. Are FastCGI daemons already running?\n");
-			return EXIT_FAILURE;
-		}
-
-		log_string("starting PHP FastCGI daemons");
-
-		/* Start daemons
-		 */
-		php_cgi = config->php_cgi;
-		while (php_cgi != NULL) {
-			if ((php_cgi->pid = run_php_cgi(php_cgi)) != -1) {
-				started++;
-			}
-			php_cgi = php_cgi->next;
-		}
-		if (started == 0) {
-			return EXIT_FAILURE;
-		}
-
-		/* Check return codes
-		 */
-		sleep(1);
-		php_cgi = config->php_cgi;
-		while (php_cgi != NULL) {
-			if (php_cgi->pid != -1) {
-				if (waitpid(php_cgi->pid, &status, WNOHANG) > 0) {
-					log_string("PHP FastCGI daemon exited with code %d", status);
-					php_cgi->pid = -1;
-				}
-			}
-			php_cgi = php_cgi->next;
-		}
-
-		/* Log PIDs
-		 */
-		if ((fp = fopen(config->pidfile, "w")) != NULL) {
-			php_cgi = config->php_cgi;
-			while (php_cgi != NULL) {
-				if (php_cgi->pid != -1) {
-					fprintf(fp, "%d\n", php_cgi->pid);
-				}
-				php_cgi = php_cgi->next;
-			}
-			fclose(fp);
-		} else {
-			fprintf(stderr, "write ");
-			perror(config->pidfile);
-		}
 	} else {
-		/* Kill PHP FastCGI servers
-		 */
-		if ((fp = fopen(config->pidfile, "r")) != NULL) {
-			log_string("stopping PHP FastCGI daemons");
-
-			line[10] = '\0';
-			while (fgets(line, 10, fp) != NULL) {
-				if ((len = strlen(line)) > 0) {
-					if (line[len - 1] == '\n') {
-						line[len - 1] = '\0';
-					}
-					if ((pid = (pid_t)str2int(line)) > 1) {
-						if (kill(pid, SIGTERM) == -1) {
-							fprintf(stderr, "kill -15 ");
-							perror(line);
-						}
-					}
-				}
-			}
-			fclose(fp);
-			if (unlink(config->pidfile) == -1) {
-				fprintf(stderr, "unlink ");
-				perror(config->pidfile);
-			}
-		} else {
-			fprintf(stderr, "read ");
-			perror(config->pidfile);
+		if (stop_fastcgi_servers(config) == -1) {
+			return EXIT_FAILURE;
 		}
 	}
 
