@@ -1,7 +1,14 @@
-/* Hiawatha | send.c
- * 
- * All the routines that send data directly to the client.
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  */
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,20 +35,22 @@
 #define MAX_TO_BUFFER 400
 #define NONCE_DIGITS 10
 #define SEND_TIMEOUT 5
+#define TIMESTR_SIZE 64
 
-char *hs_http10 = "HTTP/1.0 ";                  /*  9 */
-char *hs_http11 = "HTTP/1.1 ";                  /*  9 */
-char *hs_server = "Server: ";                   /*  8 */
-char *hs_conn   = "Connection: ";               /* 12 */
-char *hs_concl  = "close\r\n";                  /*  7 */
-char *hs_conka  = "keep-alive\r\n";             /* 12 */
-char *hs_contyp = "Content-Type: ";             /* 14 */
-char *hs_lctn   = "Location: ";                 /* 10 */
-char *hs_http   = "http://";                    /*  7 */
-char *hs_https  = "https://";                   /*  8 */
-char *hs_range  = "Accept-Ranges: bytes\r\n";   /* 22 */
-char *hs_gzip   = "Content-Encoding: gzip\r\n"; /* 24 */
-char *hs_eol    = "\r\n";                       /*  2 */
+char *hs_http10  = "HTTP/1.0 ";                  /*  9 */
+char *hs_http11  = "HTTP/1.1 ";                  /*  9 */
+char *hs_server  = "Server: ";                   /*  8 */
+char *hs_conn    = "Connection: ";               /* 12 */
+char *hs_concl   = "close\r\n";                  /*  7 */
+char *hs_conka   = "keep-alive\r\n";             /* 12 */
+char *hs_contyp  = "Content-Type: ";             /* 14 */
+char *hs_lctn    = "Location: ";                 /* 10 */
+char *hs_expires = "Expires: ";                  /*  9 */
+char *hs_http    = "http://";                    /*  7 */
+char *hs_https   = "https://";                   /*  8 */
+char *hs_range   = "Accept-Ranges: bytes\r\n";   /* 22 */
+char *hs_gzip    = "Content-Encoding: gzip\r\n"; /* 24 */
+char *hs_eol     = "\r\n";                       /*  2 */
 
 char *unknown_code = "Unknown Error";
 
@@ -162,15 +171,11 @@ int send_buffer(t_session *session, const char *buffer, int size) {
 /* Send a HTTP header to the client. Header is not closed by this function.
  */
 int send_header(t_session *session, int code) {
-	char ecode[5], timestr[64];
+	char ecode[5], timestr[TIMESTR_SIZE];
 	const char *emesg;
 	time_t t;
 	struct tm *s;
 
-	time(&t);
-	s = gmtime(&t);
-	strftime(timestr, 64, "%a, %d %b %Y %X GMT\r\n", s);
-	
 	/* Send HTTP header.
 	 */
 	ecode[4] = '\0';
@@ -181,6 +186,8 @@ int send_header(t_session *session, int code) {
 	
 	session->data_sent = true;
 
+	/* HTTP version
+	 */
 	if (session->http_version != NULL) {
 		if (*(session->http_version + 7) == '0') {
 			if (send_buffer(session, hs_http10, 9) == -1) {
@@ -197,6 +204,8 @@ int send_header(t_session *session, int code) {
 		}
 	}
 
+	/* HTTP code
+	 */
 	if (send_buffer(session, ecode, 3) == -1) {
 		return -1;
 	} else if (send_buffer(session, " ", 1) == -1) {
@@ -205,11 +214,25 @@ int send_header(t_session *session, int code) {
 		return -1;
 	} else if (send_buffer(session, hs_eol, 2) == -1) {
 		return -1;
+	}
+
+	/* Date
+	 */
+	if (time(&t) == -1) {
+		return -1;
+	} else if ((s = gmtime(&t)) == NULL) {
+		return -1;
+	} else if (strftime(timestr, TIMESTR_SIZE, "%a, %d %b %Y %X GMT\r\n", s) == 0) {
+		return -1;
 	} else if (send_buffer(session, "Date: ", 6) == -1) {
 		return -1;
 	} else if (send_buffer(session, timestr, strlen(timestr)) == -1) {
 		return -1;
-	} else if (session->config->server_string != NULL) {
+	}
+
+	/* Server
+	 */
+	if (session->config->server_string != NULL) {
 		if (send_buffer(session, hs_server, 8) == -1) {
 			return -1;
 		} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
@@ -217,11 +240,18 @@ int send_header(t_session *session, int code) {
 		} else if (send_buffer(session, hs_eol, 2) == -1) {
 			return -1;
 		}
-	} else if ((session->cgi_type == no_cgi) && (session->uri_is_dir == false)) {
+	}
+
+	/* Range
+	 */
+	if ((session->cgi_type == no_cgi) && (session->uri_is_dir == false)) {
 		if (send_buffer(session, hs_range, 22) == -1) {
 			return -1;
 		}
 	}
+
+	/* Connection
+	 */
 	if (send_buffer(session, hs_conn, 12) == -1) {
 		return -1;
 	} else if (session->keep_alive) {
@@ -231,12 +261,17 @@ int send_header(t_session *session, int code) {
 	} else if (send_buffer(session, hs_concl, 7) == -1) {
 		return -1;
 	}
+
+	/* Content-Encoding
+	 */
 	if (session->encode_gzip) {
 		if (send_buffer(session, hs_gzip, 24) == -1) {
 			return -1;
 		}
 	}
 
+	/* Content-Type
+	 */
 	if (session->mimetype != NULL) {
 		if (send_buffer(session, hs_contyp, 14) == -1) {
 			return -1;
@@ -247,6 +282,25 @@ int send_header(t_session *session, int code) {
 		}
 	}
 
+	/* Expires
+	 */
+	if ((session->expires > -1) && (code == 200)) {
+		if (time(&t) == -1) {
+			return -1;
+		}
+		t += (time_t)session->expires;
+		
+		if ((s = gmtime(&t)) == NULL) {
+			return -1;
+		} else if (send_buffer(session, hs_expires, 9) == -1) {
+			return -1;
+		} else if (strftime(timestr, TIMESTR_SIZE, "%a, %d %b %Y %X GMT\r\n", s) == 0) {
+			return -1;
+		} else if (send_buffer(session, timestr, strlen(timestr)) == -1) {
+			return -1;
+		}
+	}
+
 	return 0;
 }
 
@@ -431,9 +485,14 @@ int send_code(t_session *session, int code) {
 		return -1;
 	} else if (send_buffer(session, "text/html\r\n\r\n", 13) == -1) {
 		return -1;
-	} else if (session->request_method == HEAD) {
+	}
+	session->header_sent = true;
+
+	if (session->request_method == HEAD) {
 		return 0;
-	} else if (send_buffer(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
+	}
+	
+	if (send_buffer(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
 		return -1;
 	} else if (send_buffer(session, "<html>\n<head>\n<title>", 21) == -1) {
 		return -1;
