@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,12 +42,6 @@ enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi
 bool including = false;
 t_keyvalue *variables = NULL;
 
-/******************************************************************************
- *
- * Functions to create new records and fill them with default values.
- *
- */
-
 static t_host *new_host(void) {
 	t_host *host;
 
@@ -119,6 +124,9 @@ static t_directory *new_directory(void) {
 		directory->start_file          = NULL;
 		directory->execute_cgiset      = false;
 		directory->show_index_set      = false;
+#ifdef HAVE_XSLT
+		directory->index_xslt          = NULL;
+#endif
 		directory->use_gz_file_set     = false;
 		directory->follow_symlinks_set = false;
 		directory->access_list         = NULL;
@@ -254,10 +262,12 @@ t_config *default_config(void) {
 		config->log_format         = hiawatha;
 		config->user_directory     = "public_html";
 		config->user_directory_set = false;
+		config->hide_proxy         = NULL;
 
 		config->pidfile            = PIDFILE_DIR"/hiawatha.pid";
 		config->system_logfile     = LOG_DIR"/system.log";
 		config->garbage_logfile    = NULL;
+		config->exploit_logfile    = LOG_DIR"/exploit.log";
 		config->logfile_mask       = NULL;
 
 		config->ban_on_denied_body = 0;
@@ -303,12 +313,6 @@ t_config *default_config(void) {
 	return config;
 }
 
-/******************************************************************************
- *
- * Miscellaneous functions
- *
- */
-
 static int fgets_multi(char *line, int size, FILE *fp) {
 	int lines;
 	char *pos;
@@ -513,12 +517,6 @@ void close_bindings(t_binding *binding) {
 	}
 }
 
-/******************************************************************************
- *
- * Check configuration for validity
- *
- */
-
 int check_configuration(t_config *config) {
 	t_fcgi_server *fcgi_server;
 	t_connect_to *connect_to;
@@ -627,12 +625,6 @@ int check_configuration(t_config *config) {
 	return 0;
 }
 
-/******************************************************************************
- *
- * Parse configuration lines
- *
- */
-
 static bool system_setting(char *key, char *value, t_config *config) {
 	char *uid, *gid, *rest;
 	t_cgi_handler *cgi;
@@ -780,12 +772,22 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			return true;
 		}
 #endif
+	} else if (strcmp(key, "exploitlogfile") == 0) {
+		if (*value == '/') {
+			if ((config->exploit_logfile = strdup(value)) != NULL) {
+				return true;
+			}
+		}
 	} else if (strcmp(key, "garbagelogfile") == 0) {
 		if (*value == '/') {
 			if ((config->garbage_logfile = strdup(value)) != NULL) {
 				return true;
 			}
 		}
+	} else if (strcmp(key, "hideproxy") == 0) {
+		if (parse_iplist(value, &(config->hide_proxy)) == 0) {
+			return true;
+		}
 	} else if (strcmp(key, "kickonban") == 0) {
 		if (parse_yesno(value, &(config->kick_on_ban)) == 0) {
 			return true;
@@ -1323,6 +1325,14 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 		if (parse_yesno(value, &(directory->show_index)) == 0) {
 			directory->show_index_set = true;
 			return true;
+#ifdef HAVE_XSLT
+		} else if (*value == '/') {
+			if ((directory->index_xslt = strdup(value)) != NULL) {
+				directory->show_index = true;
+				directory->show_index_set = true;
+				return true;
+			}
+#endif
 		}
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_start_file(value)) {
@@ -1444,8 +1454,15 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 			connect_to->available = true;
 			fcgi_server->connect_to = connect_to;
 
-			if (parse_ip_port(value, &(connect_to->host), &(connect_to->port)) == -1) {
-				return false;
+			if (*value == '/') {
+				if ((connect_to->unix_socket = strdup(value)) == NULL) {
+					return false;
+				}
+			} else {
+				connect_to->unix_socket = NULL;
+				if (parse_ip_port(value, &(connect_to->host), &(connect_to->port)) == -1) {
+					return false;
+				}
 			}
 			value = rest;
 		}
@@ -1477,12 +1494,6 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 	return false;
 }
 
-/******************************************************************************
- *
- * Read functions
- *
- */
-
 static int read_config_directory(char *dir, t_config *config, bool config_check) {
 	t_filelist *filelist, *file;
 	char *path;
@@ -1869,12 +1880,6 @@ int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata)
 	return retval;
 }
 
-/******************************************************************************
- *
- * Lookup functions
- *
- */
-
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding) {
 	char *dots;
 	int i, len, len_hostname;
