@@ -1,8 +1,20 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #ifndef _SERVERCONFIG_H
 #define _SERVERCONFIG_H
 
 #include <stdbool.h>
 #include <pthread.h>
+#include <regex.h>
 #include "global.h"
 #include "libip.h"
 #include "mimetype.h"
@@ -38,6 +50,7 @@ typedef struct type_cgi_handler {
 } t_cgi_handler;
 
 typedef struct type_connect_to {
+	char          *unix_socket;
 	int           port;
 	t_ip_addr     host;
 	bool          available;
@@ -109,6 +122,9 @@ typedef struct type_directory {
 	bool          execute_cgiset;
 	bool          show_index;
 	bool          show_index_set;
+#ifdef HAVE_XSLT
+	char          *index_xslt;
+#endif
 	bool          follow_symlinks;
 	bool          follow_symlinks_set;
 	bool          use_gz_file;
@@ -228,10 +244,12 @@ typedef struct type_config {
 	bool          kill_timedout_cgi;
 	char          *system_logfile;
 	char          *garbage_logfile;
+	char          *exploit_logfile;
 	char          *pidfile;
 	t_accesslist  *logfile_mask;
 	char          *user_directory;
 	bool          user_directory_set;
+	t_iplist      *hide_proxy;
 	
 	t_mimetype    *mimetype;
 	t_host        *first_host;
