@@ -1,8 +1,14 @@
-/* Hiawatha | session.c
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
  *
- * All the routines to handle a session. A session is a connection from a
- * client to the webserver.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  */
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,6 +21,7 @@
 #include "libstr.h"
 #include "liblist.h"
 #include "session.h"
+#include "log.h"
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
@@ -42,6 +49,7 @@ static void clear_session(t_session *session) {
 	session->headerfields = NULL;
 	session->body = NULL;
 	session->local_user = NULL;
+	session->header_sent = false;
 	session->data_sent = false;
 	session->cause_of_301 = missing_slash;
 	session->header_length = 0;
@@ -69,6 +77,7 @@ static void clear_session(t_session *session) {
 	session->uploaded_file = NULL;
 	session->uploaded_size = 0;
 	session->location = NULL;
+	session->expires = -1;
 #ifdef HAVE_TOOLKIT
 	session->toolkit_fastcgi = NULL;
 #endif
@@ -218,10 +227,12 @@ int get_target_extension(t_session *session) {
 	}
 
 #ifdef CIFS
-	if ((session->extension = strdup(session->extension)) == NULL) {
-		return -1;
+	if (session->extension != NULL) {
+		if ((session->extension = strdup(session->extension)) == NULL) {
+			return -1;
+		}
+		strlower(session->extension);
 	}
-	strlower(session->extension);
 #endif
 
 	return 0;
@@ -335,6 +346,9 @@ int copy_directory_settings(t_session *session) {
 				}
 				if (dir->show_index_set) {
 					session->host->show_index = dir->show_index;
+#ifdef HAVE_XSLT
+					session->host->index_xslt = dir->index_xslt;
+#endif
 				}
 				if (dir->follow_symlinks_set) {
 					session->host->follow_symlinks = dir->follow_symlinks;
@@ -431,6 +445,96 @@ bool client_is_rejected_bot(t_session *session) {
 	return false;
 }
 
+/* Prevent cross-site scripting.
+ */
+int prevent_xss(t_session *session) {
+	int result = 0;
+	short low, high;
+	char *str, value;
+
+	if ((str = session->vars) == NULL) {
+		return 0;
+	}
+
+	while (*str != '\0') {
+		if ((value = *str) == '%') {
+			if ((high = hex_to_int(*(str + 1))) != -1) {
+				if ((low = hex_to_int(*(str + 2))) != -1) {
+					value = (char)(high<<4) + low;
+				}
+			}
+		}
+
+		if ((value == '\"') || (value == '<') || (value == '>') || (value == '\'')) {
+			if (result == 0) {
+				log_exploit_attempt(session, "XSS", session->vars);
+			}
+			*str = '_';
+			result += 1;
+		}
+		str++;
+	}
+
+	return result;
+}
+
+/* Prevent SQL injection
+ */
+int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
+	int changed = 0, i, add, left, newlength = 0;
+	char *temp, value;
+	short low, high;
+
+	if ((str == NULL) || (length == 0) || (newstr == NULL)) {
+		return 0;
+	}
+
+	add = left = 32;
+	if ((*newstr = (char*)malloc(length + add + 1)) == NULL) {
+		return -1;
+	}
+	for (i = 0; i < length; i++) {
+		value = *(str + i);
+
+		if ((value == '%') && (i + 2 < length)) {
+			if ((high = hex_to_int(*(str + i + 1))) != -1) {
+				if ((low = hex_to_int(*(str + i + 2))) != -1) {
+					value = (char)(high<<4) + low;
+				}
+			}
+		}
+
+		if ((value == '\'') || (value == '`') || (value == '\\')) {
+			if (changed == 0) {
+				log_exploit_attempt(session, "XSS", session->vars);
+			}
+
+			*(*newstr + newlength++) = '\\';
+			if (left-- == 0) {
+				add += left = 32;
+				if ((temp = (char*)realloc(*newstr, length + add + 1)) == NULL) {
+					free(*newstr);
+					return -1;
+				} else {
+					*newstr = temp;
+				}
+			}
+			changed++;
+		}
+		*(*newstr + (newlength++)) = *(str + i);
+	}
+
+	if (changed == 0) {
+		free(*newstr);
+		*newstr = NULL;
+		newlength = 0;
+	} else {
+		*(*newstr + newlength) = '\0';
+	}
+
+	return newlength;
+}
+
 /* Prevent Cross-site Request Forgery
  */
 int prevent_csrf(t_session *session) {
@@ -452,6 +556,8 @@ int prevent_csrf(t_session *session) {
 	} else {
 		session->cookie = NULL;
 
+		log_error(session, "invalid referer while checking for CSRF");
+
 		return 1;
 	}
 	
@@ -469,6 +575,12 @@ int prevent_csrf(t_session *session) {
 
 	session->cookie = NULL;
 
+	if (session->body != NULL) {
+		log_exploit_attempt(session, "CSRF", session->body);
+	} else {
+		log_error(session, "CSRF attempt detected with no request body");
+	}
+
 	return 1;
 }
 
