@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #ifndef _SESSION_H
 #define _SESSION_H
 
@@ -34,6 +45,7 @@ typedef struct type_session {
 	char            *extension;
 	char            *request_uri;
 	int             uri_len;
+	bool            header_sent;
 	bool            data_sent;
 	char            *local_user;
 	bool            force_quit;
@@ -59,6 +71,7 @@ typedef struct type_session {
 	char            *uploaded_file;
 	long            uploaded_size;
 	char            *location;
+	int             expires;
 	t_cause_of_301  cause_of_301;
 #ifdef HAVE_TOOLKIT
 	char            *toolkit_fastcgi;
@@ -107,6 +120,8 @@ bool is_volatile_object(t_session *session);
 int  copy_directory_settings(t_session *session);
 char *file_in_chroot(t_session *session, char *file);
 bool client_is_rejected_bot(t_session *session);
+int prevent_xss(t_session *session);
+int  prevent_sqli(t_session *session, char *str, int length, char **newstr);
 int  prevent_csrf(t_session *session);
 void close_socket(t_session *session);
 
