@@ -1,7 +1,14 @@
-/* Hiawatha | target.c
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
  *
- * All the routines for sending the requested item back to the client.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  */
+
 #include "config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -353,6 +360,7 @@ int send_file(t_session *session) {
 		} else if (send_buffer(session, value, strlen(value)) == -1) {
 			break;
 		}
+		session->header_sent = true;
 
 		retval = 200;
 		if (session->request_method != HEAD) {
@@ -439,7 +447,7 @@ int send_file(t_session *session) {
 /* Run a CGI program and send output to the client.
  */
 int execute_cgi(t_session *session) {
-	int retval = 200, result, handle;
+	int retval = 200, result, handle, len;
 	char *line, *new_line, *contyp_start, *contyp_end, *code, c;
 	bool in_body = false, send_in_chunks = true, wrap_cgi;
 #ifdef CYGWIN
@@ -527,7 +535,7 @@ int execute_cgi(t_session *session) {
 	if (session->host->prevent_xss) {
 		/* Prevent Cross-site Scripting
 		 */
-		prevent_xss(session->vars);
+		prevent_xss(session);
 	}
 
 	if (session->host->prevent_csrf) {
@@ -540,7 +548,7 @@ int execute_cgi(t_session *session) {
 		/* Prevent SQL injection
 		 */
 		if (session->vars != NULL) {
-			switch (prevent_sqli(session->vars, strlen(session->vars), &line)) {
+			switch (prevent_sqli(session, session->vars, strlen(session->vars), &line)) {
 				case -1:
 					return 500;
 				case 0:
@@ -559,7 +567,7 @@ int execute_cgi(t_session *session) {
 		}
 
 		if (session->body != NULL) {
-			switch (result = prevent_sqli(session->body, session->content_length, &line)) {
+			switch (result = prevent_sqli(session, session->body, session->content_length, &line)) {
 				case -1:
 					return 500;
 				case 0:
@@ -579,7 +587,7 @@ int execute_cgi(t_session *session) {
 		}
 
 		if (session->cookie != NULL) {
-			switch (prevent_sqli(session->cookie, strlen(session->cookie), &line)) {
+			switch (prevent_sqli(session, session->cookie, strlen(session->cookie), &line)) {
 				case -1:
 					return 500;
 				case 0:
@@ -708,13 +716,12 @@ int execute_cgi(t_session *session) {
 						 */
 						if (session->request_method != HEAD) {
 							if (send_in_chunks) {
-								if (send_chunk(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-									retval = rs_DISCONNECT;
-								}
+								result = send_chunk(session, cgi_info.input_buffer, cgi_info.input_len);
 							} else {
-								if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-									retval = rs_DISCONNECT;
-								}
+								result = send_buffer(session, cgi_info.input_buffer, cgi_info.input_len);
+							}
+							if (result == -1) {
+								retval = rs_DISCONNECT;
 							}
 						}
 						cgi_info.input_len = 0;
@@ -748,7 +755,13 @@ int execute_cgi(t_session *session) {
 								}
 							}
 
-							if (strcasestr(cgi_info.input_buffer, "Location:") != NULL) {
+							if (session->expires > -1) {
+								if (strcasestr(cgi_info.input_buffer, "\nExpires:") != NULL) {
+									session->expires = -1;
+								}
+							}
+
+							if (strcasestr(cgi_info.input_buffer, "\nLocation:") != NULL) {
 								if (send_header(session, 302) == -1) {
 									retval = rs_DISCONNECT;
 									break;
@@ -805,35 +818,39 @@ int execute_cgi(t_session *session) {
 
 							/* Send the header.
 							 */
-							if (send_in_chunks || (session->request_method == HEAD)) {
-								new_line += 4;
-								result = new_line - cgi_info.input_buffer;
-								if (send_buffer(session, cgi_info.input_buffer, result) == -1) {
-									retval = rs_DISCONNECT;
-									break;
-								}
-								if (send_buffer(session, NULL, 0) == -1) {
-									retval = rs_DISCONNECT;
-									break;
-								}
-								if (session->request_method != HEAD) {
-									result = cgi_info.input_len - result;
-									if (result > 0) {
-										if (send_chunk(session, new_line, result) == -1) {
-											retval = rs_DISCONNECT;
-											break;
-										}
-									}
-/*
-								} else {
-									retval = rs_QUIT;
-									break;
-*/
-								}
-							} else if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
+							new_line += 4;
+							len = new_line - cgi_info.input_buffer;
+							if (send_buffer(session, cgi_info.input_buffer, len) == -1) {
 								retval = rs_DISCONNECT;
 								break;
 							}
+							if (send_buffer(session, NULL, 0) == -1) {
+								retval = rs_DISCONNECT;
+								break;
+							}
+							session->header_sent = true;
+
+							/* Send first part of the body
+							 */
+							if (session->request_method != HEAD) {
+								if ((len = cgi_info.input_len - len) > 0) {
+									if (send_in_chunks) {
+										result = send_chunk(session, new_line, len);
+									} else {
+										result = send_buffer(session, new_line, len);
+									}
+									if (result == -1) {
+										retval = rs_DISCONNECT;
+										break;
+									}
+								}
+/*
+							} else {
+								// This will speed up things, but also disrupt the CGI process.
+								retval = rs_QUIT;
+								break;
+*/
+							}
 
 							in_body = true;
 							cgi_info.input_len = 0;
@@ -1010,7 +1027,10 @@ int show_index(t_session *session) {
 			break;
 		} else if (send_buffer(session, NULL, 0) == -1) {
 			break;
-		} else if (send_chunk(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
+		}
+		session->header_sent = true;
+
+		if (send_chunk(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
 			break;
 		} else if (send_chunk(session, "<html>\n<head>\n<title>Index of ", 30) == -1) {
 			break;
@@ -1229,6 +1249,7 @@ int handle_options_request(t_session *session) {
 	} else if (send_buffer(session, "0\r\n\r\n", 5) == -1) {
 		return -1;
 	}
+	session->header_sent = true;
 
 	return 200;
 }
@@ -1255,7 +1276,8 @@ int handle_trace_request(t_session *session) {
 	buffer[MAX_TRACE_HEADER] = '\0';
 
 	do {
-		/* Header */
+		/* Header
+		 */
 		if (snprintf(buffer, MAX_TRACE_HEADER, "%d\r\nContent-Type: message/http\r\n\r\n", body_size) < 0) {
 			break;
 		} else if (send_header(session, 200) == -1) {
@@ -1265,8 +1287,10 @@ int handle_trace_request(t_session *session) {
 		} else if (send_buffer(session, buffer, strlen(buffer)) == -1) {
 			break;
 		}
+		session->header_sent = true;
 
-		/* Body */
+		/* Body
+		 */
 		if ((code = snprintf(buffer, MAX_TRACE_HEADER, "%s %s", session->method, session->uri)) < 0) {
 			break;
 		} else if (code >= MAX_TRACE_HEADER) {
