@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #include "config.h"
 
 #ifdef HAVE_TOOLKIT
@@ -51,7 +62,7 @@ static int replace(char *src, int ofs, int len, char *rep, char **dst) {
 bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 	t_toolkit_rule *new_rule, *rule;
 	char *rest;
-	int loop;
+	int loop, time;
 
 	if ((key == NULL) || (value == NULL) || (toolkit == NULL)) {
 		return false;
@@ -92,7 +103,9 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 				/* Match UseFastCGI
 				 */
 				new_rule->action = ta_match_ban;
-				new_rule->value = str2int(rest);
+				if ((new_rule->value = str2int(rest)) == false) {
+					return false;
+				}
 			} else if (strcasecmp(value, "call") == 0) {
 				/* Match Call
 				 */
@@ -110,6 +123,47 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 				/* Match Exit
 				 */
 				new_rule->action = ta_match_exit;
+			} else if (strcasecmp(value, "expire") == 0) {
+				/* Match Expire
+				 */
+				new_rule->action = ta_match_expire;
+				if (split_string(rest, &value, &rest, ' ') == -1) {
+					return false;
+				}
+				if ((new_rule->value = str2int(value)) == -1) {
+					return false;
+				}
+
+				time = new_rule->value;
+
+				split_string(rest, &value, &rest, ' ');
+				if (strcasecmp(value, "minutes") == 0) {
+					new_rule->value *= MINUTE;
+				} else if (strcasecmp(value, "hours") == 0) {
+					new_rule->value *= HOUR;
+				} else if (strcasecmp(value, "days") == 0) {
+					new_rule->value *= DAY;
+				} else if (strcasecmp(value, "weeks") == 0) {
+					new_rule->value *= 7 * DAY;
+				} else if (strcasecmp(value, "months") == 0) {
+					new_rule->value *= 30.5 * DAY;
+				} else if (strcasecmp(value, "seconds") != 0) {
+					return false;
+				}
+
+				if (new_rule->value < time) {
+					return false;
+				}
+
+				if (rest == NULL) {
+					new_rule->done = td_continue;
+				} else if (strcasecmp(rest, "exit") == 0) {
+					new_rule->done = td_exit;
+				} else if (strcasecmp(rest, "return") == 0) {
+					new_rule->done = td_return;
+				} else {
+					return false;
+				}
 			} else if (strcasecmp(value, "goto") == 0) {
 				/* Match Goto
 				 */
@@ -336,6 +390,15 @@ static int do_rewrite(char *url, regex_t *regexp, char *rep, char **new_url, int
 	return 0;
 }
 
+void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit) {
+	options->new_url = NULL;
+	options->website_root = website_root;
+	options->fastcgi_server = NULL;
+	options->ban = 0;
+	options->expire = -1;
+	options->url_toolkit = toolkit;
+}
+
 int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 	t_url_toolkit *toolkit;
 	t_toolkit_rule *rule;
@@ -349,7 +412,6 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 	}
 
 	options->new_url = NULL;
-	options->fastcgi_server = NULL;
 
 	if ((toolkit = find_toolkit(toolkit_id, options->url_toolkit)) == NULL) {
 		return UT_ERROR;
@@ -427,6 +489,8 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 					}
 					url = options->new_url;
 					replaced = true;
+				} else if (replaced) {
+					options->new_url = url;
 				}
 
 				if (result == UT_EXIT) {
@@ -451,6 +515,17 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 					return UT_EXIT;
 				}
 				break;
+			case ta_match_expire:
+				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
+					options->expire = rule->value;
+
+					if (rule->done == td_return) {
+						return UT_RETURN;
+					} else if (rule->done == td_exit) {
+						return UT_EXIT;
+					}
+				}
+				break;
 			case ta_match_fastcgi:
 				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
 					options->fastcgi_server = rule->parameter;
